@@ -24,13 +24,14 @@ func TestGitCommandProxy(t *testing.T) {
 
 	os.Setenv("http_proxy", ts.URL)
 
-	ctx, _ := context.WithCancel(context.Background())
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
 
 	dir, err := ioutil.TempDir("", "test-clone")
 	require.NoError(t, err)
 	defer os.RemoveAll(dir)
 
-	cmd, err := CommandWithoutRepo(ctx, "clone", "http://gitlab.com/bogus-repo", dir)
+	cmd, err := unsafeCmdWithoutRepo(ctx, CmdStream{}, "clone", "http://gitlab.com/bogus-repo", dir)
 	require.NoError(t, err)
 
 	err = cmd.Wait()
