@@ -1,13 +1,11 @@
 package commit
 
 import (
-	"gitlab.com/gitlab-org/gitaly/internal/rubyserver"
 	"gitlab.com/gitlab-org/gitaly/internal/service/ref"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
 
 type server struct {
-	ruby *rubyserver.Server
 	gitalypb.UnimplementedCommitServiceServer
 }
 
@@ -16,6 +14,6 @@ var (
 )
 
 // NewServer creates a new instance of a grpc CommitServiceServer
-func NewServer(rs *rubyserver.Server) gitalypb.CommitServiceServer {
-	return &server{ruby: rs}
+func NewServer() gitalypb.CommitServiceServer {
+	return &server{}
 }
