@@ -1,7 +1,6 @@
 package diff
 
 import (
-	"gitlab.com/gitlab-org/gitaly/internal/rubyserver"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
 
@@ -9,14 +8,12 @@ const msgSizeThreshold = 5 * 1024
 
 type server struct {
 	MsgSizeThreshold int
-	ruby             *rubyserver.Server
 	gitalypb.UnimplementedDiffServiceServer
 }
 
 // NewServer creates a new instance of a gRPC DiffServer
-func NewServer(rs *rubyserver.Server) gitalypb.DiffServiceServer {
+func NewServer() gitalypb.DiffServiceServer {
 	return &server{
 		MsgSizeThreshold: msgSizeThreshold,
-		ruby:             rs,
 	}
 }
