@@ -1,9 +1,9 @@
 package operations
 
 import (
+	"context"
 	"fmt"
 	"io/ioutil"
-	"os"
 	"os/exec"
 	"strings"
 	"testing"
@@ -18,12 +18,6 @@ import (
 )
 
 var (
-	mergeUser = &gitalypb.User{
-		Name:  []byte("Jane Doe"),
-		Email: []byte("janedoe@example.com"),
-		GlId:  "user-1",
-	}
-
 	commitToMerge         = "e63f41fe459e62e1228fcef60d7189127aeba95a"
 	mergeBranchName       = "gitaly-merge-test-branch"
 	mergeBranchHeadBefore = "281d3a76f31c812dbf48abce82ccf6860adedd81"
@@ -33,11 +27,15 @@ func TestSuccessfulMerge(t *testing.T) {
 	ctx, cancel := testhelper.Context()
 	defer cancel()
 
+	testSuccessfulMerge(t, ctx)
+}
+
+func testSuccessfulMerge(t *testing.T, ctx context.Context) {
 	testRepo, testRepoPath, cleanupFn := testhelper.NewTestRepo(t)
 	defer cleanupFn()
 
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -50,14 +48,15 @@ func TestSuccessfulMerge(t *testing.T) {
 	hooks := GitlabHooks
 	hookTempfiles := make([]string, len(hooks))
 	for i, h := range hooks {
-		hookTempfiles[i] = WriteEnvToHook(t, testRepoPath, h)
-		defer os.Remove(hookTempfiles[i])
+		var cleanup func()
+		hookTempfiles[i], cleanup = testhelper.WriteEnvToCustomHook(t, testRepoPath, h)
+		defer cleanup()
 	}
 
 	mergeCommitMessage := "Merged by Gitaly"
 	firstRequest := &gitalypb.UserMergeBranchRequest{
 		Repository: testRepo,
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 		CommitId:   commitToMerge,
 		Branch:     []byte(mergeBranchName),
 		Message:    []byte(mergeCommitMessage),
@@ -93,10 +92,10 @@ func TestSuccessfulMerge(t *testing.T) {
 	require.True(t, strings.HasPrefix(string(commit.Body), mergeCommitMessage), "expected %q to start with %q", commit.Body, mergeCommitMessage)
 
 	author := commit.Author
-	require.Equal(t, mergeUser.Name, author.Name)
-	require.Equal(t, mergeUser.Email, author.Email)
+	require.Equal(t, testhelper.TestUser.Name, author.Name)
+	require.Equal(t, testhelper.TestUser.Email, author.Email)
 
-	expectedGlID := "GL_ID=" + mergeUser.GlId
+	expectedGlID := "GL_ID=" + testhelper.TestUser.GlId
 	for i, h := range hooks {
 		hookEnv, err := ioutil.ReadFile(hookTempfiles[i])
 		require.NoError(t, err)
@@ -108,8 +107,8 @@ func TestAbortedMerge(t *testing.T) {
 	ctx, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -121,7 +120,7 @@ func TestAbortedMerge(t *testing.T) {
 
 	firstRequest := &gitalypb.UserMergeBranchRequest{
 		Repository: testRepo,
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 		CommitId:   commitToMerge,
 		Branch:     []byte(mergeBranchName),
 		Message:    []byte("foobar"),
@@ -179,8 +178,8 @@ func TestFailedMergeConcurrentUpdate(t *testing.T) {
 	testRepo, testRepoPath, cleanupFn := testhelper.NewTestRepo(t)
 	defer cleanupFn()
 
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -193,7 +192,7 @@ func TestFailedMergeConcurrentUpdate(t *testing.T) {
 	mergeCommitMessage := "Merged by Gitaly"
 	firstRequest := &gitalypb.UserMergeBranchRequest{
 		Repository: testRepo,
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 		CommitId:   commitToMerge,
 		Branch:     []byte(mergeBranchName),
 		Message:    []byte(mergeCommitMessage),
@@ -212,7 +211,7 @@ func TestFailedMergeConcurrentUpdate(t *testing.T) {
 
 	secondResponse, err := mergeBidi.Recv()
 	require.NoError(t, err, "receive second response")
-	require.Equal(t, *secondResponse, gitalypb.UserMergeBranchResponse{}, "response should be empty")
+	testhelper.ProtoEqual(t, secondResponse, &gitalypb.UserMergeBranchResponse{})
 
 	commit, err := gitlog.GetCommit(ctx, testRepo, mergeBranchName)
 	require.NoError(t, err, "get commit after RPC finished")
@@ -223,8 +222,8 @@ func TestFailedMergeDueToHooks(t *testing.T) {
 	testRepo, testRepoPath, cleanupFn := testhelper.NewTestRepo(t)
 	defer cleanupFn()
 
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -235,7 +234,7 @@ func TestFailedMergeDueToHooks(t *testing.T) {
 
 	for _, hookName := range gitlabPreHooks {
 		t.Run(hookName, func(t *testing.T) {
-			remove, err := OverrideHooks(hookName, hookContent)
+			remove, err := testhelper.WriteCustomHook(testRepoPath, hookName, hookContent)
 			require.NoError(t, err)
 			defer remove()
 
@@ -248,7 +247,7 @@ func TestFailedMergeDueToHooks(t *testing.T) {
 			mergeCommitMessage := "Merged by Gitaly"
 			firstRequest := &gitalypb.UserMergeBranchRequest{
 				Repository: testRepo,
-				User:       mergeUser,
+				User:       testhelper.TestUser,
 				CommitId:   commitToMerge,
 				Branch:     []byte(mergeBranchName),
 				Message:    []byte(mergeCommitMessage),
@@ -282,8 +281,8 @@ func TestSuccessfulUserFFBranchRequest(t *testing.T) {
 	ctx, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -297,7 +296,7 @@ func TestSuccessfulUserFFBranchRequest(t *testing.T) {
 		Repository: testRepo,
 		CommitId:   commitID,
 		Branch:     []byte(branchName),
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 	}
 	expectedResponse := &gitalypb.UserFFBranchResponse{
 		BranchUpdate: &gitalypb.OperationBranchUpdate{
@@ -312,14 +311,14 @@ func TestSuccessfulUserFFBranchRequest(t *testing.T) {
 
 	resp, err := client.UserFFBranch(ctx, request)
 	require.NoError(t, err)
-	require.Equal(t, expectedResponse, resp)
+	testhelper.ProtoEqual(t, expectedResponse, resp)
 	newBranchHead := testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "rev-parse", branchName)
 	require.Equal(t, commitID, text.ChompBytes(newBranchHead), "branch head not updated")
 }
 
 func TestFailedUserFFBranchRequest(t *testing.T) {
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -343,7 +342,7 @@ func TestFailedUserFFBranchRequest(t *testing.T) {
 	}{
 		{
 			desc:     "empty repository",
-			user:     mergeUser,
+			user:     testhelper.TestUser,
 			branch:   []byte(branchName),
 			commitID: commitID,
 			code:     codes.InvalidArgument,
@@ -358,14 +357,14 @@ func TestFailedUserFFBranchRequest(t *testing.T) {
 		{
 			desc:   "empty commit",
 			repo:   testRepo,
-			user:   mergeUser,
+			user:   testhelper.TestUser,
 			branch: []byte(branchName),
 			code:   codes.InvalidArgument,
 		},
 		{
 			desc:     "non-existing commit",
 			repo:     testRepo,
-			user:     mergeUser,
+			user:     testhelper.TestUser,
 			branch:   []byte(branchName),
 			commitID: "f001",
 			code:     codes.InvalidArgument,
@@ -373,14 +372,14 @@ func TestFailedUserFFBranchRequest(t *testing.T) {
 		{
 			desc:     "empty branch",
 			repo:     testRepo,
-			user:     mergeUser,
+			user:     testhelper.TestUser,
 			commitID: commitID,
 			code:     codes.InvalidArgument,
 		},
 		{
 			desc:     "non-existing branch",
 			repo:     testRepo,
-			user:     mergeUser,
+			user:     testhelper.TestUser,
 			branch:   []byte("this-isnt-real"),
 			commitID: commitID,
 			code:     codes.InvalidArgument,
@@ -388,7 +387,7 @@ func TestFailedUserFFBranchRequest(t *testing.T) {
 		{
 			desc:     "commit is not a descendant of branch head",
 			repo:     testRepo,
-			user:     mergeUser,
+			user:     testhelper.TestUser,
 			branch:   []byte(branchName),
 			commitID: "1a0b36b3cdad1d2ee32457c102a8c0b7056fa863",
 			code:     codes.FailedPrecondition,
@@ -413,8 +412,8 @@ func TestFailedUserFFBranchRequest(t *testing.T) {
 }
 
 func TestFailedUserFFBranchDueToHooks(t *testing.T) {
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -428,7 +427,7 @@ func TestFailedUserFFBranchDueToHooks(t *testing.T) {
 		Repository: testRepo,
 		CommitId:   commitID,
 		Branch:     []byte(branchName),
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 	}
 
 	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "branch", "-f", branchName, "6d394385cf567f80a8fd85055db1ab4c5295806f")
@@ -438,7 +437,7 @@ func TestFailedUserFFBranchDueToHooks(t *testing.T) {
 
 	for _, hookName := range gitlabPreHooks {
 		t.Run(hookName, func(t *testing.T) {
-			remove, err := OverrideHooks(hookName, hookContent)
+			remove, err := testhelper.WriteCustomHook(testRepoPath, hookName, hookContent)
 			require.NoError(t, err)
 			defer remove()
 
@@ -453,8 +452,8 @@ func TestFailedUserFFBranchDueToHooks(t *testing.T) {
 }
 
 func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -485,7 +484,7 @@ func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
 	}{
 		{
 			desc:           "empty target ref merge",
-			user:           mergeUser,
+			user:           testhelper.TestUser,
 			targetRef:      emptyTargetRef,
 			emptyRef:       true,
 			sourceSha:      commitToMerge,
@@ -494,7 +493,7 @@ func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
 		},
 		{
 			desc:           "existing target ref",
-			user:           mergeUser,
+			user:           testhelper.TestUser,
 			targetRef:      existingTargetRef,
 			emptyRef:       false,
 			sourceSha:      commitToMerge,
@@ -503,7 +502,7 @@ func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
 		},
 		{
 			desc:      "branch is specified and firstParentRef is empty",
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte(mergeBranchName),
 			targetRef: existingTargetRef,
 			emptyRef:  false,
@@ -547,8 +546,8 @@ func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
 
 			// Asserts author
 			author := commit.Author
-			require.Equal(t, mergeUser.Name, author.Name)
-			require.Equal(t, mergeUser.Email, author.Email)
+			require.Equal(t, testhelper.TestUser.Name, author.Name)
+			require.Equal(t, testhelper.TestUser.Email, author.Email)
 
 			require.Equal(t, resp.CommitId, commit.Id)
 
@@ -562,8 +561,8 @@ func TestSuccessfulUserMergeToRefRequest(t *testing.T) {
 }
 
 func TestFailedUserMergeToRefRequest(t *testing.T) {
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -586,7 +585,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 	}{
 		{
 			desc:      "empty repository",
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte(branchName),
 			sourceSha: commitToMerge,
 			targetRef: validTargetRef,
@@ -603,7 +602,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 		{
 			desc:      "empty source SHA",
 			repo:      testRepo,
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte(branchName),
 			targetRef: validTargetRef,
 			code:      codes.InvalidArgument,
@@ -611,7 +610,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 		{
 			desc:      "non-existing commit",
 			repo:      testRepo,
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte(branchName),
 			sourceSha: "f001",
 			targetRef: validTargetRef,
@@ -620,7 +619,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 		{
 			desc:      "empty branch and first parent ref",
 			repo:      testRepo,
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			sourceSha: commitToMerge,
 			targetRef: validTargetRef,
 			code:      codes.InvalidArgument,
@@ -628,7 +627,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 		{
 			desc:      "invalid target ref",
 			repo:      testRepo,
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte(branchName),
 			sourceSha: commitToMerge,
 			targetRef: []byte("refs/heads/branch"),
@@ -637,7 +636,7 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 		{
 			desc:      "non-existing branch",
 			repo:      testRepo,
-			user:      mergeUser,
+			user:      testhelper.TestUser,
 			branch:    []byte("this-isnt-real"),
 			sourceSha: commitToMerge,
 			targetRef: validTargetRef,
@@ -664,8 +663,8 @@ func TestFailedUserMergeToRefRequest(t *testing.T) {
 }
 
 func TestUserMergeToRefIgnoreHooksRequest(t *testing.T) {
-	server, serverSocketPath := runOperationServiceServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
 	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
@@ -683,7 +682,7 @@ func TestUserMergeToRefIgnoreHooksRequest(t *testing.T) {
 		SourceSha:  commitToMerge,
 		Branch:     []byte(mergeBranchName),
 		TargetRef:  targetRef,
-		User:       mergeUser,
+		User:       testhelper.TestUser,
 		Message:    []byte(mergeCommitMessage),
 	}
 
@@ -691,7 +690,7 @@ func TestUserMergeToRefIgnoreHooksRequest(t *testing.T) {
 
 	for _, hookName := range gitlabPreHooks {
 		t.Run(hookName, func(t *testing.T) {
-			remove, err := OverrideHooks(hookName, hookContent)
+			remove, err := testhelper.WriteCustomHook(testRepoPath, hookName, hookContent)
 			require.NoError(t, err)
 			defer remove()
 
