@@ -1,9 +1,9 @@
-package operations_test
+package operations
 
 //lint:file-ignore SA1019 due to planned removal in issue https://gitlab.com/gitlab-org/gitaly/issues/1628
 
 import (
-	"errors"
+	"context"
 	"fmt"
 	"strings"
 	"testing"
@@ -11,7 +11,8 @@ import (
 
 	"github.com/stretchr/testify/require"
 	gitlog "gitlab.com/gitlab-org/gitaly/internal/git/log"
-	"gitlab.com/gitlab-org/gitaly/internal/service/operations"
+	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
+	"gitlab.com/gitlab-org/gitaly/internal/rubyserver"
 	"gitlab.com/gitlab-org/gitaly/internal/testhelper"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc/codes"
@@ -19,23 +20,38 @@ import (
 )
 
 var (
-	rebaseUser = &gitalypb.User{
-		Name:  []byte("Ahmad Sherif"),
-		Email: []byte("ahmad@gitlab.com"),
-		GlId:  "user-123",
-	}
-
-	branchName = "many_files"
+	rebaseBranchName = "many_files"
 )
 
 func TestSuccessfulUserRebaseConfirmableRequest(t *testing.T) {
-	ctxOuter, cancel := testhelper.Context()
+	featureSets, err := testhelper.NewFeatureSets(nil, featureflag.GoPostReceiveHook)
+	require.NoError(t, err)
+
+	ctx, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	var ruby rubyserver.Server
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	pushOptions := []string{"ci.skip", "test=value"}
+	cleanupSrv := setupAndStartGitlabServer(t, testhelper.GlID, "project-1", pushOptions...)
+	defer cleanupSrv()
+
+	require.NoError(t, ruby.Start())
+	defer ruby.Stop()
+
+	serverSocketPath, stop := runOperationServiceServerWithRubyServer(t, &ruby)
+	defer stop()
+
+	for _, featureSet := range featureSets {
+		t.Run(featureSet.String(), func(t *testing.T) {
+			ctx := featureSet.WithParent(ctx)
+			testSuccessfulUserRebaseConfirmableRequest(t, ctx, serverSocketPath, pushOptions)
+		})
+	}
+}
+
+func testSuccessfulUserRebaseConfirmableRequest(t *testing.T, ctxOuter context.Context, serverSocketPath string, pushOptions []string) {
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
@@ -44,15 +60,27 @@ func TestSuccessfulUserRebaseConfirmableRequest(t *testing.T) {
 	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
 	defer cleanup()
 
-	branchSha := getBranchSha(t, testRepoPath, branchName)
+	branchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
 	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
+
+	mdFromCtx, ok := metadata.FromOutgoingContext(ctxOuter)
+	if ok {
+		md = metadata.Join(md, mdFromCtx)
+	}
+
 	ctx := metadata.NewOutgoingContext(ctxOuter, md)
 
 	rebaseStream, err := client.UserRebaseConfirmable(ctx)
 	require.NoError(t, err)
 
-	headerRequest := buildHeaderRequest(testRepo, rebaseUser, "1", branchName, branchSha, testRepoCopy, "master")
+	preReceiveHookOutputPath, removePreReceive := testhelper.WriteEnvToCustomHook(t, testRepoPath, "pre-receive")
+	postReceiveHookOutputPath, removePostReceive := testhelper.WriteEnvToCustomHook(t, testRepoPath, "post-receive")
+	defer removePreReceive()
+	defer removePostReceive()
+
+	headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, branchSha, testRepoCopy, "master")
+	headerRequest.GetHeader().GitPushOptions = pushOptions
 	require.NoError(t, rebaseStream.Send(headerRequest), "send header")
 
 	firstResponse, err := rebaseStream.Recv()
@@ -73,22 +101,29 @@ func TestSuccessfulUserRebaseConfirmableRequest(t *testing.T) {
 	})
 	require.NoError(t, err, "consume EOF")
 
-	newBranchSha := getBranchSha(t, testRepoPath, branchName)
+	newBranchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
 	require.NotEqual(t, newBranchSha, branchSha)
 	require.Equal(t, newBranchSha, firstResponse.GetRebaseSha())
 
 	require.True(t, secondResponse.GetRebaseApplied(), "the second rebase is applied")
+
+	for _, outputPath := range []string{preReceiveHookOutputPath, postReceiveHookOutputPath} {
+		output := string(testhelper.MustReadFile(t, outputPath))
+		require.Contains(t, output, "GIT_PUSH_OPTION_COUNT=2")
+		require.Contains(t, output, "GIT_PUSH_OPTION_0=ci.skip")
+		require.Contains(t, output, "GIT_PUSH_OPTION_1=test=value")
+	}
 }
 
 func TestFailedRebaseUserRebaseConfirmableRequestDueToInvalidHeader(t *testing.T) {
 	ctxOuter, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
@@ -97,7 +132,7 @@ func TestFailedRebaseUserRebaseConfirmableRequestDueToInvalidHeader(t *testing.T
 	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
 	defer cleanup()
 
-	branchSha := getBranchSha(t, testRepoPath, branchName)
+	branchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
 	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
 	ctx := metadata.NewOutgoingContext(ctxOuter, md)
@@ -108,27 +143,31 @@ func TestFailedRebaseUserRebaseConfirmableRequestDueToInvalidHeader(t *testing.T
 	}{
 		{
 			desc: "empty Repository",
-			req:  buildHeaderRequest(nil, rebaseUser, "1", branchName, branchSha, testRepoCopy, "master"),
+			req:  buildHeaderRequest(nil, testhelper.TestUser, "1", rebaseBranchName, branchSha, testRepoCopy, "master"),
 		},
 		{
 			desc: "empty User",
-			req:  buildHeaderRequest(testRepo, nil, "1", branchName, branchSha, testRepoCopy, "master"),
+			req:  buildHeaderRequest(testRepo, nil, "1", rebaseBranchName, branchSha, testRepoCopy, "master"),
 		},
 		{
 			desc: "empty Branch",
-			req:  buildHeaderRequest(testRepo, rebaseUser, "1", "", branchSha, testRepoCopy, "master"),
+			req:  buildHeaderRequest(testRepo, testhelper.TestUser, "1", "", branchSha, testRepoCopy, "master"),
 		},
 		{
 			desc: "empty BranchSha",
-			req:  buildHeaderRequest(testRepo, rebaseUser, "1", branchName, "", testRepoCopy, "master"),
+			req:  buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, "", testRepoCopy, "master"),
 		},
 		{
 			desc: "empty RemoteRepository",
-			req:  buildHeaderRequest(testRepo, rebaseUser, "1", branchName, branchSha, nil, "master"),
+			req:  buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, branchSha, nil, "master"),
 		},
 		{
 			desc: "empty RemoteBranch",
-			req:  buildHeaderRequest(testRepo, rebaseUser, "1", branchName, branchSha, testRepoCopy, ""),
+			req:  buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, branchSha, testRepoCopy, ""),
+		},
+		{
+			desc: "invalid branch name",
+			req:  buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, branchSha, testRepoCopy, "+dev:master"),
 		},
 	}
 
@@ -151,10 +190,10 @@ func TestAbortedUserRebaseConfirmable(t *testing.T) {
 	ctxOuter, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
@@ -179,9 +218,9 @@ func TestAbortedUserRebaseConfirmable(t *testing.T) {
 			testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
 			defer cleanup()
 
-			branchSha := getBranchSha(t, testRepoPath, branchName)
+			branchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
-			headerRequest := buildHeaderRequest(testRepo, rebaseUser, fmt.Sprintf("%v", i), branchName, branchSha, testRepoCopy, "master")
+			headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, fmt.Sprintf("%v", i), rebaseBranchName, branchSha, testRepoCopy, "master")
 
 			rebaseStream, err := client.UserRebaseConfirmable(ctx)
 			require.NoError(t, err)
@@ -200,7 +239,7 @@ func TestAbortedUserRebaseConfirmable(t *testing.T) {
 				require.NoError(t, rebaseStream.CloseSend(), "close request stream from client")
 			}
 
-			secondResponse, err := recvTimeout(rebaseStream, 1*time.Second)
+			secondResponse, err := rebaseRecvTimeout(rebaseStream, 1*time.Second)
 			if err == errRecvTimeout {
 				t.Fatal(err)
 			}
@@ -209,7 +248,7 @@ func TestAbortedUserRebaseConfirmable(t *testing.T) {
 			require.Error(t, err)
 			testhelper.RequireGrpcError(t, err, tc.code)
 
-			newBranchSha := getBranchSha(t, testRepoPath, branchName)
+			newBranchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 			require.Equal(t, newBranchSha, branchSha, "branch should not change when the rebase is aborted")
 		})
 	}
@@ -219,10 +258,10 @@ func TestFailedUserRebaseConfirmableDueToApplyBeingFalse(t *testing.T) {
 	ctxOuter, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
@@ -231,7 +270,7 @@ func TestFailedUserRebaseConfirmableDueToApplyBeingFalse(t *testing.T) {
 	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
 	defer cleanup()
 
-	branchSha := getBranchSha(t, testRepoPath, branchName)
+	branchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
 	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
 	ctx := metadata.NewOutgoingContext(ctxOuter, md)
@@ -239,7 +278,7 @@ func TestFailedUserRebaseConfirmableDueToApplyBeingFalse(t *testing.T) {
 	rebaseStream, err := client.UserRebaseConfirmable(ctx)
 	require.NoError(t, err)
 
-	headerRequest := buildHeaderRequest(testRepo, rebaseUser, "1", branchName, branchSha, testRepoCopy, "master")
+	headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, "1", rebaseBranchName, branchSha, testRepoCopy, "master")
 	require.NoError(t, rebaseStream.Send(headerRequest), "send header")
 
 	firstResponse, err := rebaseStream.Recv()
@@ -256,7 +295,7 @@ func TestFailedUserRebaseConfirmableDueToApplyBeingFalse(t *testing.T) {
 	testhelper.RequireGrpcError(t, err, codes.FailedPrecondition)
 	require.False(t, secondResponse.GetRebaseApplied(), "the second rebase is not applied")
 
-	newBranchSha := getBranchSha(t, testRepoPath, branchName)
+	newBranchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 	require.Equal(t, branchSha, newBranchSha, "branch should not change when the rebase is not applied")
 	require.NotEqual(t, newBranchSha, firstResponse.GetRebaseSha(), "branch should not be the sha returned when the rebase is not applied")
 }
@@ -265,10 +304,10 @@ func TestFailedUserRebaseConfirmableRequestDueToPreReceiveError(t *testing.T) {
 	ctxOuter, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
@@ -277,13 +316,13 @@ func TestFailedUserRebaseConfirmableRequestDueToPreReceiveError(t *testing.T) {
 	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
 	defer cleanup()
 
-	branchSha := getBranchSha(t, testRepoPath, branchName)
+	branchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 
 	hookContent := []byte("#!/bin/sh\necho 'failure'\nexit 1")
 
-	for i, hookName := range operations.GitlabPreHooks {
+	for i, hookName := range GitlabPreHooks {
 		t.Run(hookName, func(t *testing.T) {
-			remove, err := operations.OverrideHooks(hookName, hookContent)
+			remove, err := testhelper.WriteCustomHook(testRepoPath, hookName, hookContent)
 			require.NoError(t, err, "set up hooks override")
 			defer remove()
 
@@ -293,7 +332,7 @@ func TestFailedUserRebaseConfirmableRequestDueToPreReceiveError(t *testing.T) {
 			rebaseStream, err := client.UserRebaseConfirmable(ctx)
 			require.NoError(t, err)
 
-			headerRequest := buildHeaderRequest(testRepo, rebaseUser, fmt.Sprintf("%v", i), branchName, branchSha, testRepoCopy, "master")
+			headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, fmt.Sprintf("%v", i), rebaseBranchName, branchSha, testRepoCopy, "master")
 			require.NoError(t, rebaseStream.Send(headerRequest), "send header")
 
 			firstResponse, err := rebaseStream.Recv()
@@ -316,7 +355,7 @@ func TestFailedUserRebaseConfirmableRequestDueToPreReceiveError(t *testing.T) {
 			})
 			require.NoError(t, err, "consume EOF")
 
-			newBranchSha := getBranchSha(t, testRepoPath, branchName)
+			newBranchSha := getBranchSha(t, testRepoPath, rebaseBranchName)
 			require.Equal(t, branchSha, newBranchSha, "branch should not change when the rebase fails due to PreReceiveError")
 			require.NotEqual(t, newBranchSha, firstResponse.GetRebaseSha(), "branch should not be the sha returned when the rebase fails due to PreReceiveError")
 		})
@@ -327,10 +366,10 @@ func TestFailedUserRebaseConfirmableDueToGitError(t *testing.T) {
 	ctxOuter, cancel := testhelper.Context()
 	defer cancel()
 
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
 
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
+	client, conn := newOperationClient(t, serverSocketPath)
 	defer conn.Close()
 
 	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
@@ -348,12 +387,12 @@ func TestFailedUserRebaseConfirmableDueToGitError(t *testing.T) {
 	rebaseStream, err := client.UserRebaseConfirmable(ctx)
 	require.NoError(t, err)
 
-	headerRequest := buildHeaderRequest(testRepo, rebaseUser, "1", failedBranchName, branchSha, testRepoCopy, "master")
+	headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, "1", failedBranchName, branchSha, testRepoCopy, "master")
 	require.NoError(t, rebaseStream.Send(headerRequest), "send header")
 
 	firstResponse, err := rebaseStream.Recv()
 	require.NoError(t, err, "receive first response")
-	require.Contains(t, firstResponse.GitError, "error: Failed to merge in the changes.")
+	require.Contains(t, firstResponse.GitError, "CONFLICT (content): Merge conflict in README.md")
 
 	err = testhelper.ReceiveEOFWithTimeout(func() error {
 		_, err = rebaseStream.Recv()
@@ -365,266 +404,73 @@ func TestFailedUserRebaseConfirmableDueToGitError(t *testing.T) {
 	require.Equal(t, branchSha, newBranchSha, "branch should not change when the rebase fails due to GitError")
 }
 
-// DEPRECATED: https://gitlab.com/gitlab-org/gitaly/issues/1628
-func TestSuccessfulUserRebaseRequest(t *testing.T) {
-	ctxOuter, cancel := testhelper.Context()
-	defer cancel()
-
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
-
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
-	defer conn.Close()
-
-	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	branchSha := getBranchSha(t, testRepoPath, branchName)
-
-	request := &gitalypb.UserRebaseRequest{
-		Repository:       testRepo,
-		User:             rebaseUser,
-		RebaseId:         "1",
-		Branch:           []byte(branchName),
-		BranchSha:        branchSha,
-		RemoteRepository: testRepoCopy,
-		RemoteBranch:     []byte("master"),
-	}
-
-	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
-	ctx := metadata.NewOutgoingContext(ctxOuter, md)
-
-	response, err := client.UserRebase(ctx, request)
-	require.NoError(t, err)
-
-	newBranchSha := getBranchSha(t, testRepoPath, branchName)
-
-	require.NotEqual(t, newBranchSha, branchSha)
-	require.Equal(t, newBranchSha, response.RebaseSha)
-}
-
-// DEPRECATED: https://gitlab.com/gitlab-org/gitaly/issues/1628
-func TestFailedUserRebaseRequestDueToPreReceiveError(t *testing.T) {
-	ctxOuter, cancel := testhelper.Context()
-	defer cancel()
-
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
-
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
-	defer conn.Close()
-
-	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	branchSha := getBranchSha(t, testRepoPath, branchName)
-
-	request := &gitalypb.UserRebaseRequest{
-		Repository:       testRepo,
-		User:             rebaseUser,
-		Branch:           []byte(branchName),
-		BranchSha:        branchSha,
-		RemoteRepository: testRepoCopy,
-		RemoteBranch:     []byte("master"),
-	}
-
-	hookContent := []byte("#!/bin/sh\necho GL_ID=$GL_ID\nexit 1\n")
-	for i, hookName := range operations.GitlabPreHooks {
-		t.Run(hookName, func(t *testing.T) {
-			remove, err := operations.OverrideHooks(hookName, hookContent)
-			require.NoError(t, err, "set up hooks override")
-			defer remove()
-
-			md := testhelper.GitalyServersMetadata(t, serverSocketPath)
-			ctx := metadata.NewOutgoingContext(ctxOuter, md)
-
-			request.RebaseId = fmt.Sprintf("%d", i+1)
-			response, err := client.UserRebase(ctx, request)
-			require.NoError(t, err)
-			require.Contains(t, response.PreReceiveError, "GL_ID="+rebaseUser.GlId)
-		})
-	}
-}
-
-// DEPRECATED: https://gitlab.com/gitlab-org/gitaly/issues/1628
-func TestFailedUserRebaseRequestDueToGitError(t *testing.T) {
-	ctxOuter, cancel := testhelper.Context()
-	defer cancel()
-
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
-
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
-	defer conn.Close()
-
-	testRepo, testRepoPath, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	failedBranchName := "rebase-encoding-failure-trigger"
-	branchSha := getBranchSha(t, testRepoPath, failedBranchName)
-
-	request := &gitalypb.UserRebaseRequest{
-		Repository:       testRepo,
-		User:             rebaseUser,
-		RebaseId:         "1",
-		Branch:           []byte(failedBranchName),
-		BranchSha:        branchSha,
-		RemoteRepository: testRepoCopy,
-		RemoteBranch:     []byte("master"),
-	}
-
-	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
-	ctx := metadata.NewOutgoingContext(ctxOuter, md)
-
-	response, err := client.UserRebase(ctx, request)
-	require.NoError(t, err)
-	require.Contains(t, response.GitError, "error: Failed to merge in the changes.")
-}
-
-// DEPRECATED: https://gitlab.com/gitlab-org/gitaly/issues/1628
-func TestFailedUserRebaseRequestDueToValidations(t *testing.T) {
-	ctxOuter, cancel := testhelper.Context()
-	defer cancel()
-
-	server, serverSocketPath := runFullServer(t)
-	defer server.Stop()
-
-	client, conn := operations.NewOperationClient(t, serverSocketPath)
-	defer conn.Close()
-
-	testRepo, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
-	defer cleanup()
-
-	testCases := []struct {
-		desc    string
-		request *gitalypb.UserRebaseRequest
-		code    codes.Code
-	}{
-		{
-			desc: "empty repository",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       nil,
-				User:             rebaseUser,
-				RebaseId:         "1",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty user",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             nil,
-				RebaseId:         "1",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty rebase id",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             rebaseUser,
-				RebaseId:         "",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty branch",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             rebaseUser,
-				RebaseId:         "1",
-				Branch:           nil,
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty branch sha",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             rebaseUser,
-				RebaseId:         "1",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty remote repository",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             rebaseUser,
-				RebaseId:         "1",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: nil,
-				RemoteBranch:     []byte("master"),
-			},
-			code: codes.InvalidArgument,
-		},
-		{
-			desc: "empty remote branch",
-			request: &gitalypb.UserRebaseRequest{
-				Repository:       testRepo,
-				User:             rebaseUser,
-				RebaseId:         "1",
-				Branch:           []byte("some-branch"),
-				BranchSha:        "38008cb17ce1466d8fec2dfa6f6ab8dcfe5cf49e",
-				RemoteRepository: testRepoCopy,
-				RemoteBranch:     nil,
-			},
-			code: codes.InvalidArgument,
-		},
-	}
-
-	for _, testCase := range testCases {
-		t.Run(testCase.desc, func(t *testing.T) {
-			md := testhelper.GitalyServersMetadata(t, serverSocketPath)
-			ctx := metadata.NewOutgoingContext(ctxOuter, md)
-
-			_, err := client.UserRebase(ctx, testCase.request)
-			testhelper.RequireGrpcError(t, err, testCase.code)
-		})
-	}
-}
-
 func getBranchSha(t *testing.T, repoPath string, branchName string) string {
 	branchSha := string(testhelper.MustRunCommand(t, nil, "git", "-C", repoPath, "rev-parse", branchName))
 	return strings.TrimSpace(branchSha)
 }
 
-// This error is used as a sentinel value
-var errRecvTimeout = errors.New("timeout waiting for response")
+func TestRebaseRequestWithDeletedFile(t *testing.T) {
+	ctxOuter, cancel := testhelper.Context()
+	defer cancel()
 
-func recvTimeout(bidi gitalypb.OperationService_UserRebaseConfirmableClient, timeout time.Duration) (*gitalypb.UserRebaseConfirmableResponse, error) {
+	serverSocketPath, stop := runOperationServiceServer(t)
+	defer stop()
+
+	client, conn := newOperationClient(t, serverSocketPath)
+	defer conn.Close()
+
+	testRepo, testRepoPath, cleanupFn := testhelper.NewTestRepoWithWorktree(t)
+	defer cleanupFn()
+
+	testRepoCopy, _, cleanup := testhelper.NewTestRepo(t)
+	defer cleanup()
+
+	md := testhelper.GitalyServersMetadata(t, serverSocketPath)
+	ctx := metadata.NewOutgoingContext(ctxOuter, md)
+
+	branch := "rebase-delete-test"
+
+	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "config", "user.name", string(testhelper.TestUser.Name))
+	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "config", "user.email", string(testhelper.TestUser.Email))
+	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "checkout", "-b", branch, "master~1")
+	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "rm", "README")
+	testhelper.MustRunCommand(t, nil, "git", "-C", testRepoPath, "commit", "-a", "-m", "delete file")
+
+	branchSha := getBranchSha(t, testRepoPath, branch)
+
+	rebaseStream, err := client.UserRebaseConfirmable(ctx)
+	require.NoError(t, err)
+
+	headerRequest := buildHeaderRequest(testRepo, testhelper.TestUser, "1", branch, branchSha, testRepoCopy, "master")
+	require.NoError(t, rebaseStream.Send(headerRequest), "send header")
+
+	firstResponse, err := rebaseStream.Recv()
+	require.NoError(t, err, "receive first response")
+
+	_, err = gitlog.GetCommit(ctx, testRepo, firstResponse.GetRebaseSha())
+	require.NoError(t, err, "look up git commit before rebase is applied")
+
+	applyRequest := buildApplyRequest(true)
+	require.NoError(t, rebaseStream.Send(applyRequest), "apply rebase")
+
+	secondResponse, err := rebaseStream.Recv()
+	require.NoError(t, err, "receive second response")
+
+	err = testhelper.ReceiveEOFWithTimeout(func() error {
+		_, err = rebaseStream.Recv()
+		return err
+	})
+	require.NoError(t, err, "consume EOF")
+
+	newBranchSha := getBranchSha(t, testRepoPath, branch)
+
+	require.NotEqual(t, newBranchSha, branchSha)
+	require.Equal(t, newBranchSha, firstResponse.GetRebaseSha())
+
+	require.True(t, secondResponse.GetRebaseApplied(), "the second rebase is applied")
+}
+
+func rebaseRecvTimeout(bidi gitalypb.OperationService_UserRebaseConfirmableClient, timeout time.Duration) (*gitalypb.UserRebaseConfirmableResponse, error) {
 	type responseError struct {
 		response *gitalypb.UserRebaseConfirmableResponse
 		err      error
@@ -644,13 +490,13 @@ func recvTimeout(bidi gitalypb.OperationService_UserRebaseConfirmableClient, tim
 	}
 }
 
-func buildHeaderRequest(repo *gitalypb.Repository, user *gitalypb.User, rebaseId string, branchName string, branchSha string, remoteRepo *gitalypb.Repository, remoteBranch string) *gitalypb.UserRebaseConfirmableRequest {
+func buildHeaderRequest(repo *gitalypb.Repository, user *gitalypb.User, rebaseID string, branchName string, branchSha string, remoteRepo *gitalypb.Repository, remoteBranch string) *gitalypb.UserRebaseConfirmableRequest {
 	return &gitalypb.UserRebaseConfirmableRequest{
 		UserRebaseConfirmableRequestPayload: &gitalypb.UserRebaseConfirmableRequest_Header_{
-			&gitalypb.UserRebaseConfirmableRequest_Header{
+			Header: &gitalypb.UserRebaseConfirmableRequest_Header{
 				Repository:       repo,
 				User:             user,
-				RebaseId:         rebaseId,
+				RebaseId:         rebaseID,
 				Branch:           []byte(branchName),
 				BranchSha:        branchSha,
 				RemoteRepository: remoteRepo,
