@@ -33,7 +33,7 @@ func validatePackRefsRequest(in *gitalypb.PackRefsRequest) error {
 func packRefs(ctx context.Context, repository repository.GitRepo, all bool) error {
 	cmd, err := git.SafeCmd(ctx, repository, nil, git.SubCmd{
 		Name:  "pack-refs",
-		Flags: []git.Option{git.Flag{"--all"}},
+		Flags: []git.Option{git.Flag{Name: "--all"}},
 	})
 	if err != nil {
 		return fmt.Errorf("initializing pack-refs command: %v", err)
