@@ -6,7 +6,6 @@ import (
 	"path"
 
 	"gitlab.com/gitlab-org/gitaly/internal/command"
-	"gitlab.com/gitlab-org/gitaly/internal/helper"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"gitlab.com/gitlab-org/gitaly/streamio"
 	"google.golang.org/grpc/codes"
@@ -16,7 +15,7 @@ import (
 const customHooksDir = "custom_hooks"
 
 func (s *server) BackupCustomHooks(in *gitalypb.BackupCustomHooksRequest, stream gitalypb.RepositoryService_BackupCustomHooksServer) error {
-	repoPath, err := helper.GetPath(in.Repository)
+	repoPath, err := s.locator.GetPath(in.Repository)
 	if err != nil {
 		return status.Errorf(codes.Internal, "BackupCustomHooks: getting repo path failed %v", err)
 	}
