@@ -3,6 +3,7 @@ package ssh
 import (
 	"time"
 
+	"github.com/prometheus/client_golang/prometheus"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 )
 
@@ -14,6 +15,7 @@ var (
 type server struct {
 	uploadPackRequestTimeout    time.Duration
 	uploadArchiveRequestTimeout time.Duration
+	packfileNegotiationMetrics  *prometheus.CounterVec
 	gitalypb.UnimplementedSSHServiceServer
 }
 
@@ -22,6 +24,10 @@ func NewServer(serverOpts ...ServerOpt) gitalypb.SSHServiceServer {
 	s := &server{
 		uploadPackRequestTimeout:    defaultUploadPackRequestTimeout,
 		uploadArchiveRequestTimeout: defaultUploadArchiveRequestTimeout,
+		packfileNegotiationMetrics: prometheus.NewCounterVec(
+			prometheus.CounterOpts{},
+			[]string{"git_negotiation_feature"},
+		),
 	}
 
 	for _, serverOpt := range serverOpts {
@@ -47,3 +53,9 @@ func WithArchiveRequestTimeout(d time.Duration) ServerOpt {
 		s.uploadArchiveRequestTimeout = d
 	}
 }
+
+func WithPackfileNegotiationMetrics(c *prometheus.CounterVec) ServerOpt {
+	return func(s *server) {
+		s.packfileNegotiationMetrics = c
+	}
+}
