@@ -2,17 +2,20 @@ package testhelper
 
 import (
 	"io/ioutil"
+	"log"
 	"os"
+	"path"
 	"path/filepath"
 	"testing"
 
 	"github.com/stretchr/testify/require"
+	"gitlab.com/gitlab-org/gitaly/internal/config"
 	"gitlab.com/gitlab-org/gitaly/internal/git/hooks"
 )
 
 // CaptureHookEnv creates a bogus 'update' Git hook to sniff out what
 // environment variables get set for hooks.
-func CaptureHookEnv(t *testing.T) (hookPath string, cleanup func()) {
+func CaptureHookEnv(t testing.TB) (hookPath string, cleanup func()) {
 	var err error
 	oldOverride := hooks.Override
 	hooks.Override, err = filepath.Abs("testdata/scratch/hooks")
@@ -32,3 +35,18 @@ env | grep -e ^GIT -e ^GL_ > `+hookOutputFile+"\n"), 0755))
 		hooks.Override = oldOverride
 	}
 }
+
+// ConfigureGitalyHooksBinary builds gitaly-hooks command for tests
+func ConfigureGitalyHooksBinary() {
+	if config.Config.BinDir == "" {
+		log.Fatal("config.Config.BinDir must be set")
+	}
+
+	goBuildArgs := []string{
+		"build",
+		"-o",
+		path.Join(config.Config.BinDir, "gitaly-hooks"),
+		"gitlab.com/gitlab-org/gitaly/cmd/gitaly-hooks",
+	}
+	MustRunCommand(nil, nil, "go", goBuildArgs...)
+}
