@@ -3,18 +3,27 @@ package testhelper
 import (
 	"bytes"
 	"context"
+	"crypto/ecdsa"
+	"crypto/elliptic"
+	"crypto/rand"
+	"crypto/sha256"
+	"crypto/x509"
 	"encoding/base64"
 	"encoding/json"
+	"encoding/pem"
 	"fmt"
 	"io"
 	"io/ioutil"
+	"math/big"
 	"net"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
 	"runtime"
+	"strconv"
 	"strings"
+	"sync"
 	"syscall"
 	"testing"
 	"time"
@@ -30,6 +39,7 @@ import (
 	"gitlab.com/gitlab-org/gitaly/internal/helper/fieldextractors"
 	"gitlab.com/gitlab-org/gitaly/internal/helper/text"
 	gitalylog "gitlab.com/gitlab-org/gitaly/internal/log"
+	"gitlab.com/gitlab-org/gitaly/internal/metadata/featureflag"
 	"gitlab.com/gitlab-org/gitaly/internal/storage"
 	"gitlab.com/gitlab-org/gitaly/proto/go/gitalypb"
 	"google.golang.org/grpc"
@@ -44,45 +54,63 @@ const (
 	RepositoryAuthToken = "the-secret-token"
 	DefaultStorageName  = "default"
 	testGitEnv          = "testdata/git-env"
+	GlRepository        = "project-1"
+	GlID                = "user-123"
 )
 
-func init() {
-	if err := configure(); err != nil {
-		log.Fatal(err)
+var configureOnce sync.Once
+
+var (
+	TestUser = &gitalypb.User{
+		Name:       []byte("Jane Doe"),
+		Email:      []byte("janedoe@gitlab.com"),
+		GlId:       GlID,
+		GlUsername: "janedoe",
 	}
-}
+)
 
-func configure() error {
-	config.Config.Storages = []config.Storage{
-		{Name: "default", Path: GitlabTestStoragePath()},
-	}
+// Configure sets up the global test configuration. On failure,
+// terminates the program.
+func Configure() {
+	configureOnce.Do(func() {
+		gitalylog.Configure("json", "info")
 
-	config.Config.SocketPath = "/bogus"
-	config.Config.GitlabShell.Dir = "/"
-
-	dir, err := ioutil.TempDir("", "internal_socket")
-	if err != nil {
-		return err
-	}
-
-	config.Config.InternalSocketDir = dir
-
-	for _, f := range []func() error{
-		ConfigureRuby,
-		config.Validate,
-	} {
-		if err := f(); err != nil {
-			return err
+		config.Config.Storages = []config.Storage{
+			{Name: "default", Path: GitlabTestStoragePath()},
 		}
-	}
 
-	gitalylog.Configure("", "info")
+		config.Config.SocketPath = "/bogus"
+		config.Config.GitlabShell.Dir = "/"
 
-	return nil
+		dir, err := ioutil.TempDir("", "internal_socket")
+		if err != nil {
+			log.Fatalf("error configuring tests: %v", err)
+		}
+
+		config.Config.InternalSocketDir = dir
+
+		if err := os.MkdirAll("testdata/gitaly-libexec", 0755); err != nil {
+			log.Fatal(err)
+		}
+		config.Config.BinDir, err = filepath.Abs("testdata/gitaly-libexec")
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		for _, f := range []func() error{
+			ConfigureRuby,
+			ConfigureGit,
+			config.Validate,
+		} {
+			if err := f(); err != nil {
+				log.Fatalf("error configuring tests: %v", err)
+			}
+		}
+	})
 }
 
 // MustReadFile returns the content of a file or fails at once.
-func MustReadFile(t *testing.T, filename string) []byte {
+func MustReadFile(t testing.TB, filename string) []byte {
 	content, err := ioutil.ReadFile(filename)
 	if err != nil {
 		t.Fatal(err)
@@ -93,17 +121,6 @@ func MustReadFile(t *testing.T, filename string) []byte {
 
 // GitlabTestStoragePath returns the storage path to the gitlab-test repo.
 func GitlabTestStoragePath() string {
-	// If TEST_REPO_STORAGE_PATH has been set (by the Makefile) then use that
-	testRepoPath := os.Getenv("TEST_REPO_STORAGE_PATH")
-	if testRepoPath != "" {
-		testRepoPathAbs, err := filepath.Abs(testRepoPath)
-		if err != nil {
-			log.Fatal(err)
-		}
-
-		return testRepoPathAbs
-	}
-
 	_, currentFile, _, ok := runtime.Caller(0)
 	if !ok {
 		log.Fatal("Could not get caller info")
@@ -113,7 +130,7 @@ func GitlabTestStoragePath() string {
 
 // GitalyServersMetadata returns a metadata pair for gitaly-servers to be used in
 // inter-gitaly operations.
-func GitalyServersMetadata(t *testing.T, serverSocketPath string) metadata.MD {
+func GitalyServersMetadata(t testing.TB, serverSocketPath string) metadata.MD {
 	gitalyServers := storage.GitalyServers{
 		"default": {
 			"address": serverSocketPath,
@@ -129,9 +146,9 @@ func GitalyServersMetadata(t *testing.T, serverSocketPath string) metadata.MD {
 	return metadata.Pairs("gitaly-servers", base64.StdEncoding.EncodeToString(gitalyServersJSON))
 }
 
-func testRepoValid(repo *gitalypb.Repository) bool {
-	storagePath, _ := config.Config.StoragePath(repo.GetStorageName())
-	if _, err := os.Stat(path.Join(storagePath, repo.RelativePath, "objects")); err != nil {
+// isValidRepoPath checks whether a valid git repository exists at the given path.
+func isValidRepoPath(absolutePath string) bool {
+	if _, err := os.Stat(filepath.Join(absolutePath, "objects")); err != nil {
 		return false
 	}
 
@@ -146,10 +163,11 @@ func TestRepository() *gitalypb.Repository {
 	repo := &gitalypb.Repository{
 		StorageName:  "default",
 		RelativePath: TestRelativePath,
-		GlRepository: "project-1",
+		GlRepository: GlRepository,
 	}
 
-	if !testRepoValid(repo) {
+	storagePath, _ := config.Config.StoragePath(repo.GetStorageName())
+	if !isValidRepoPath(filepath.Join(storagePath, repo.RelativePath)) {
 		panic("Test repo not found, did you run `make prepare-tests`?")
 	}
 
@@ -157,7 +175,9 @@ func TestRepository() *gitalypb.Repository {
 }
 
 // RequireGrpcError asserts the passed err is of the same code as expectedCode.
-func RequireGrpcError(t *testing.T, err error, expectedCode codes.Code) {
+func RequireGrpcError(t testing.TB, err error, expectedCode codes.Code) {
+	t.Helper()
+
 	if err == nil {
 		t.Fatal("Expected an error, got nil")
 	}
@@ -291,6 +311,34 @@ func GetLocalhostListener(t testing.TB) (net.Listener, string) {
 	return l, addr
 }
 
+// ConfigureGit configures git for test purpose
+func ConfigureGit() error {
+	_, currentFile, _, ok := runtime.Caller(0)
+	if !ok {
+		return fmt.Errorf("could not get caller info")
+	}
+
+	// Set both GOCACHE and GOPATH to the currently active settings to not
+	// have them be overridden by changing our home directory. default it
+	for _, envvar := range []string{"GOCACHE", "GOPATH"} {
+		cmd := exec.Command("go", "env", envvar)
+
+		output, err := cmd.Output()
+		if err != nil {
+			return err
+		}
+
+		err = os.Setenv(envvar, strings.TrimSpace(string(output)))
+		if err != nil {
+			return err
+		}
+	}
+
+	testHome := filepath.Join(filepath.Dir(currentFile), "testdata/home")
+	// overwrite HOME env variable so user global .gitconfig doesn't influence tests
+	return os.Setenv("HOME", testHome)
+}
+
 // ConfigureRuby configures Ruby settings for test purposes at run time.
 func ConfigureRuby() error {
 	if dir := os.Getenv("GITALY_TEST_RUBY_DIR"); len(dir) > 0 {
@@ -396,31 +444,30 @@ func Context() (context.Context, func()) {
 }
 
 // CreateRepo creates a temporary directory for a repo, without initializing it
-func CreateRepo(t testing.TB, storagePath string) (repo *gitalypb.Repository, repoPath, relativePath string) {
-	normalizedPrefix := strings.Replace(t.Name(), "/", "-", -1) //TempDir doesn't like a prefix containing slashes
-
-	repoPath, err := ioutil.TempDir(storagePath, normalizedPrefix)
-	require.NoError(t, err)
-
-	relativePath, err = filepath.Rel(storagePath, repoPath)
-	require.NoError(t, err)
-	repo = &gitalypb.Repository{StorageName: "default", RelativePath: relativePath, GlRepository: "project-1"}
-
-	return repo, repoPath, relativePath
+func CreateRepo(t testing.TB, storagePath, relativePath string) *gitalypb.Repository {
+	require.NoError(t, os.MkdirAll(filepath.Dir(storagePath), 0755), "making repo parent dir")
+	return &gitalypb.Repository{
+		StorageName:  "default",
+		RelativePath: relativePath,
+		GlRepository: GlRepository,
+	}
 }
 
 // InitBareRepo creates a new bare repository
-func InitBareRepo(t *testing.T) (*gitalypb.Repository, string, func()) {
+func InitBareRepo(t testing.TB) (*gitalypb.Repository, string, func()) {
 	return initRepo(t, true)
 }
 
 // InitRepoWithWorktree creates a new repository with a worktree
-func InitRepoWithWorktree(t *testing.T) (*gitalypb.Repository, string, func()) {
+func InitRepoWithWorktree(t testing.TB) (*gitalypb.Repository, string, func()) {
 	return initRepo(t, false)
 }
 
-func initRepo(t *testing.T, bare bool) (*gitalypb.Repository, string, func()) {
-	repo, repoPath, _ := CreateRepo(t, GitlabTestStoragePath())
+func initRepo(t testing.TB, bare bool) (*gitalypb.Repository, string, func()) {
+	storagePath := GitlabTestStoragePath()
+	relativePath := NewRepositoryName(t, bare)
+	repoPath := filepath.Join(storagePath, relativePath)
+
 	args := []string{"init"}
 	if bare {
 		args = append(args, "--bare")
@@ -428,31 +475,47 @@ func initRepo(t *testing.T, bare bool) (*gitalypb.Repository, string, func()) {
 
 	MustRunCommand(t, nil, "git", append(args, repoPath)...)
 
+	repo := CreateRepo(t, storagePath, relativePath)
 	if !bare {
 		repo.RelativePath = path.Join(repo.RelativePath, ".git")
 	}
 
-	return repo, repoPath, func() { os.RemoveAll(repoPath) }
+	return repo, repoPath, func() { require.NoError(t, os.RemoveAll(repoPath)) }
 }
 
-// NewTestRepo creates a bare copy of the test repository.
+// NewTestRepoTo clones a new copy of test repository under a subdirectory in the storage root.
+func NewTestRepoTo(t testing.TB, storageRoot, relativePath string) *gitalypb.Repository {
+	repo, _, _ := cloneTestRepo(t, storageRoot, relativePath, true)
+	return repo
+}
+
+// NewTestRepo creates a bare copy of the test repository..
 func NewTestRepo(t testing.TB) (repo *gitalypb.Repository, repoPath string, cleanup func()) {
-	return cloneTestRepo(t, true)
+	return cloneTestRepo(t, GitlabTestStoragePath(), NewRepositoryName(t, true), true)
 }
 
 // NewTestRepoWithWorktree creates a copy of the test repository with a
 // worktree. This is allows you to run normal 'non-bare' Git commands.
-func NewTestRepoWithWorktree(t *testing.T) (repo *gitalypb.Repository, repoPath string, cleanup func()) {
-	return cloneTestRepo(t, false)
+func NewTestRepoWithWorktree(t testing.TB) (repo *gitalypb.Repository, repoPath string, cleanup func()) {
+	return cloneTestRepo(t, GitlabTestStoragePath(), NewRepositoryName(t, false), false)
 }
 
-func cloneTestRepo(t testing.TB, bare bool) (repo *gitalypb.Repository, repoPath string, cleanup func()) {
-	storagePath := GitlabTestStoragePath()
-	repo, repoPath, relativePath := CreateRepo(t, storagePath)
-	testRepo := TestRepository()
-	testRepoPath := path.Join(storagePath, testRepo.RelativePath)
-	args := []string{"clone", "--no-hardlinks", "--dissociate"}
+// testRepositoryPath returns the absolute path of local 'gitlab-org/gitlab-test.git' clone.
+// It is cloned under the path by the test preparing step of make.
+func testRepositoryPath(t testing.TB) string {
+	path := filepath.Join(GitlabTestStoragePath(), TestRelativePath)
+	if !isValidRepoPath(path) {
+		t.Fatalf("local clone of 'gitlab-org/gitlab-test.git' not found in %q, did you run `make prepare-tests`?", path)
+	}
 
+	return path
+}
+
+func cloneTestRepo(t testing.TB, storageRoot, relativePath string, bare bool) (repo *gitalypb.Repository, repoPath string, cleanup func()) {
+	repoPath = filepath.Join(storageRoot, relativePath)
+
+	repo = CreateRepo(t, storageRoot, relativePath)
+	args := []string{"clone", "--no-hardlinks", "--dissociate"}
 	if bare {
 		args = append(args, "--bare")
 	} else {
@@ -460,9 +523,9 @@ func cloneTestRepo(t testing.TB, bare bool) (repo *gitalypb.Repository, repoPath
 		repo.RelativePath = path.Join(relativePath, ".git")
 	}
 
-	MustRunCommand(t, nil, "git", append(args, testRepoPath, repoPath)...)
+	MustRunCommand(t, nil, "git", append(args, testRepositoryPath(t), repoPath)...)
 
-	return repo, repoPath, func() { os.RemoveAll(repoPath) }
+	return repo, repoPath, func() { require.NoError(t, os.RemoveAll(repoPath)) }
 }
 
 // AddWorktreeArgs returns git command arguments for adding a worktree at the
@@ -472,17 +535,14 @@ func AddWorktreeArgs(repoPath, worktreeName string) []string {
 }
 
 // AddWorktree creates a worktree in the repository path for tests
-func AddWorktree(t *testing.T, repoPath string, worktreeName string) {
+func AddWorktree(t testing.TB, repoPath string, worktreeName string) {
 	MustRunCommand(t, nil, "git", AddWorktreeArgs(repoPath, worktreeName)...)
 }
 
 // ConfigureGitalySSH configures the gitaly-ssh command for tests
 func ConfigureGitalySSH() {
-	var err error
-
-	config.Config.BinDir, err = filepath.Abs("testdata/gitaly-libexec")
-	if err != nil {
-		log.Fatal(err)
+	if config.Config.BinDir == "" {
+		log.Fatal("config.Config.BinDir must be set")
 	}
 
 	goBuildArgs := []string{
@@ -495,40 +555,64 @@ func ConfigureGitalySSH() {
 }
 
 // GetRepositoryRefs gives a list of each repository ref as a string
-func GetRepositoryRefs(t *testing.T, repoPath string) string {
+func GetRepositoryRefs(t testing.TB, repoPath string) string {
 	refs := MustRunCommand(t, nil, "git", "-C", repoPath, "for-each-ref")
 
 	return string(refs)
 }
 
 // AssertPathNotExists asserts true if the path doesn't exist, false otherwise
-func AssertPathNotExists(t *testing.T, path string) {
+func AssertPathNotExists(t testing.TB, path string) {
 	_, err := os.Stat(path)
 	assert.True(t, os.IsNotExist(err), "file should not exist: %s", path)
 }
 
-// NewTestObjectPoolName returns a random pool repository name.
-func NewTestObjectPoolName(t *testing.T) string {
-	b, err := text.RandomHex(5)
+// newDiskHash generates a random directory path following the Rails app's
+// approach in the hashed storage module, formatted as '[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{64}'.
+// https://gitlab.com/gitlab-org/gitlab/-/blob/f5c7d8eb1dd4eee5106123e04dec26d277ff6a83/app/models/storage/hashed.rb#L38-43
+func newDiskHash(t testing.TB) string {
+	// rails app calculates a sha256 and uses its hex representation
+	// as the directory path
+	b, err := text.RandomHex(sha256.Size)
 	require.NoError(t, err)
+	return filepath.Join(b[0:2], b[2:4], b)
+}
 
-	return fmt.Sprintf("pool-%s.git", b)
+// NewRepositoryName returns a random repository hash
+// in format '@hashed/[0-9a-f]{2}/[0-9a-f]{2}/[0-9a-f]{64}(.git)?'.
+func NewRepositoryName(t testing.TB, bare bool) string {
+	suffix := ""
+	if bare {
+		suffix = ".git"
+	}
+
+	return filepath.Join("@hashed", newDiskHash(t)+suffix)
+}
+
+// NewTestObjectPoolName returns a random pool repository name
+// in format '@pools/[0-9a-z]{2}/[0-9a-z]{2}/[0-9a-z]{64}.git'.
+func NewTestObjectPoolName(t testing.TB) string {
+	return filepath.Join("@pools", newDiskHash(t)+".git")
 }
 
 // CreateLooseRef creates a ref that points to master
-func CreateLooseRef(t *testing.T, repoPath, refName string) {
+func CreateLooseRef(t testing.TB, repoPath, refName string) {
 	relRefPath := fmt.Sprintf("refs/heads/%s", refName)
 	MustRunCommand(t, nil, "git", "-C", repoPath, "update-ref", relRefPath, "master")
 	require.FileExists(t, filepath.Join(repoPath, relRefPath), "ref must be in loose file")
 }
 
-// TempDir is a wrapper around ioutil.TempDir that provides a cleanup function
-func TempDir(t *testing.T, dir, prefix string) (string, func() error) {
-	dirPath, err := ioutil.TempDir(dir, prefix)
-	require.NoError(t, err)
-	return dirPath, func() error {
-		return os.RemoveAll(dirPath)
+// TempDir is a wrapper around ioutil.TempDir that provides a cleanup function.
+func TempDir(t testing.TB) (string, func()) {
+	_, currentFile, _, ok := runtime.Caller(0)
+	if !ok {
+		log.Fatal("Could not get caller info")
 	}
+
+	rootTmpDir := path.Join(path.Dir(currentFile), "testdata/tmp")
+	tmpDir, err := ioutil.TempDir(rootTmpDir, "")
+	require.NoError(t, err)
+	return tmpDir, func() { require.NoError(t, os.RemoveAll(tmpDir)) }
 }
 
 // GitObjectMustExist is a test assertion that fails unless the git repo in repoPath contains sha
@@ -543,6 +627,10 @@ func GitObjectMustNotExist(t testing.TB, repoPath, sha string) {
 
 func gitObjectExists(t testing.TB, repoPath, sha string, exists bool) {
 	cmd := exec.Command("git", "-C", repoPath, "cat-file", "-e", sha)
+	cmd.Env = []string{
+		"GIT_ALLOW_PROTOCOL=", // To prevent partial clone reaching remote repo over SSH
+	}
+
 	if exists {
 		require.NoError(t, cmd.Run(), "checking for object should succeed")
 		return
@@ -553,3 +641,253 @@ func gitObjectExists(t testing.TB, repoPath, sha string, exists bool) {
 // Cleanup functions should be called in a defer statement
 // immediately after they are returned from a test helper
 type Cleanup func()
+
+// GetGitObjectDirSize gets the number of 1k blocks of a git object directory
+func GetGitObjectDirSize(t testing.TB, repoPath string) int64 {
+	return getGitDirSize(t, repoPath, "objects")
+}
+
+// GetGitPackfileDirSize gets the number of 1k blocks of a git object directory
+func GetGitPackfileDirSize(t testing.TB, repoPath string) int64 {
+	return getGitDirSize(t, repoPath, "objects", "pack")
+}
+
+func getGitDirSize(t testing.TB, repoPath string, subdirs ...string) int64 {
+	cmd := exec.Command("du", "-s", "-k", filepath.Join(append([]string{repoPath}, subdirs...)...))
+	output, err := cmd.Output()
+	require.NoError(t, err)
+	if len(output) < 2 {
+		t.Error("invalid output of du -s -k")
+	}
+
+	outputSplit := strings.SplitN(string(output), "\t", 2)
+	blocks, err := strconv.ParseInt(outputSplit[0], 10, 64)
+	require.NoError(t, err)
+
+	return blocks
+}
+
+func GrpcErrorHasMessage(grpcError error, msg string) bool {
+	status, ok := status.FromError(grpcError)
+	if !ok {
+		return false
+	}
+	return status.Message() == msg
+}
+
+// dump the env vars that the custom hooks receives to a file
+func WriteEnvToCustomHook(t testing.TB, repoPath, hookName string) (string, func()) {
+	hookOutputTemp, err := ioutil.TempFile("", "")
+	require.NoError(t, err)
+	require.NoError(t, hookOutputTemp.Close())
+
+	hookContent := fmt.Sprintf("#!/bin/sh\n/usr/bin/env > %s\n", hookOutputTemp.Name())
+
+	cleanupCustomHook, err := WriteCustomHook(repoPath, hookName, []byte(hookContent))
+	require.NoError(t, err)
+
+	return hookOutputTemp.Name(), func() {
+		cleanupCustomHook()
+		os.Remove(hookOutputTemp.Name())
+	}
+}
+
+// write a hook in the repo/path.git/custom_hooks directory
+func WriteCustomHook(repoPath, name string, content []byte) (func(), error) {
+	fullPath := filepath.Join(repoPath, "custom_hooks", name)
+	return WriteExecutable(fullPath, content)
+}
+
+// WriteExecutable ensures that the parent directory exists, and writes an executable with provided content
+func WriteExecutable(path string, content []byte) (func(), error) {
+	dir := filepath.Dir(path)
+	if err := os.MkdirAll(dir, 0755); err != nil {
+		return func() {}, err
+	}
+
+	return func() {
+		os.RemoveAll(dir)
+	}, ioutil.WriteFile(path, content, 0755)
+}
+
+// CheckNewObjectExists is a script meant to be used as a pre-receive custom hook.
+// It only succeeds if it can find the object in the quarantine directory.
+// if GIT_OBJECT_DIRECTORY and GIT_ALTERNATE_OBJECT_DIRECTORIES were not passed through correctly to the hooks,
+// it will fail
+const CheckNewObjectExists = `#!/usr/bin/env ruby
+STDIN.each_line do |line|
+  new_object = line.split(' ')[1]
+  exit 1 unless new_object
+  exit 1 unless	system(*%W[git cat-file -e #{new_object}])
+end
+`
+
+func WriteBlobs(t testing.TB, testRepoPath string, n int) []string {
+	var blobIDs []string
+	for i := 0; i < n; i++ {
+		var stdin bytes.Buffer
+		stdin.Write([]byte(strconv.Itoa(time.Now().Nanosecond())))
+		blobIDs = append(blobIDs, text.ChompBytes(MustRunCommand(t, &stdin, "git", "-C", testRepoPath, "hash-object", "-w", "--stdin")))
+	}
+
+	return blobIDs
+}
+
+// FeatureSet is a representation of a set of features that are enabled
+// This is useful in situations where a test needs to test any combination of features toggled on and off
+type FeatureSet struct {
+	features     map[featureflag.FeatureFlag]bool
+	rubyFeatures map[featureflag.FeatureFlag]bool
+}
+
+func (f FeatureSet) IsEnabled(flag featureflag.FeatureFlag) bool {
+	on, ok := f.features[flag]
+	if !ok {
+		return flag.OnByDefault
+	}
+
+	return on
+}
+
+func (f FeatureSet) String() string {
+	features := make([]string, 0, len(f.enabledFeatures()))
+	for _, feature := range f.enabledFeatures() {
+		features = append(features, feature.Name)
+	}
+
+	return strings.Join(features, ",")
+}
+
+func (f FeatureSet) enabledFeatures() []featureflag.FeatureFlag {
+	var enabled []featureflag.FeatureFlag
+
+	for feature := range f.features {
+		enabled = append(enabled, feature)
+	}
+
+	return enabled
+}
+
+func (f FeatureSet) WithParent(ctx context.Context) context.Context {
+	for _, enabledFeature := range f.enabledFeatures() {
+		if _, ok := f.rubyFeatures[enabledFeature]; ok {
+			ctx = featureflag.OutgoingCtxWithRubyFeatureFlags(ctx, enabledFeature)
+			continue
+		}
+		ctx = featureflag.OutgoingCtxWithFeatureFlags(ctx, enabledFeature)
+	}
+
+	return ctx
+}
+
+// FeatureSets is a slice containing many FeatureSets
+type FeatureSets []FeatureSet
+
+// NewFeatureSets takes a slice of go feature flags, and an optional variadic set of ruby feature flags
+// and returns a FeatureSets slice
+func NewFeatureSets(goFeatures []featureflag.FeatureFlag, rubyFeatures ...featureflag.FeatureFlag) (FeatureSets, error) {
+	rubyFeatureMap := make(map[featureflag.FeatureFlag]bool)
+	for _, rubyFeature := range rubyFeatures {
+		rubyFeatureMap[rubyFeature] = true
+	}
+
+	// start with an empty feature set
+	f := []FeatureSet{{features: make(map[featureflag.FeatureFlag]bool), rubyFeatures: rubyFeatureMap}}
+
+	allFeatures := append(goFeatures, rubyFeatures...)
+
+	for i := range allFeatures {
+		featureMap := make(map[featureflag.FeatureFlag]bool)
+		for j := 0; j <= i; j++ {
+			featureMap[allFeatures[j]] = true
+		}
+
+		f = append(f, FeatureSet{features: featureMap, rubyFeatures: rubyFeatureMap})
+	}
+
+	return f, nil
+}
+
+// ModifyEnvironment will change an environment variable and return a func suitable
+// for `defer` to change the value back.
+func ModifyEnvironment(t testing.TB, key string, value string) func() {
+	t.Helper()
+
+	oldValue, hasOldValue := os.LookupEnv(key)
+	require.NoError(t, os.Setenv(key, value))
+	return func() {
+		if hasOldValue {
+			require.NoError(t, os.Setenv(key, oldValue))
+		} else {
+			require.NoError(t, os.Unsetenv(key))
+		}
+	}
+}
+
+// GenerateTestCerts creates a certificate that can be used to establish TLS protected TCP connection.
+// It returns paths to the file with the certificate and its private key.
+func GenerateTestCerts(t *testing.T) (string, string, Cleanup) {
+	t.Helper()
+
+	rootCA := &x509.Certificate{
+		SerialNumber:          big.NewInt(1),
+		NotBefore:             time.Now(),
+		NotAfter:              time.Now().AddDate(0, 0, 1),
+		BasicConstraintsValid: true,
+		IsCA:                  true,
+		IPAddresses:           []net.IP{net.ParseIP("0.0.0.0"), net.ParseIP("127.0.0.1"), net.ParseIP("::1"), net.ParseIP("::")},
+		DNSNames:              []string{"localhost"},
+		KeyUsage:              x509.KeyUsageCertSign,
+	}
+
+	caKey, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
+	require.NoError(t, err)
+
+	caCert, err := x509.CreateCertificate(rand.Reader, rootCA, rootCA, &caKey.PublicKey, caKey)
+	require.NoError(t, err)
+
+	entityKey, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
+	require.NoError(t, err)
+
+	entityX509 := &x509.Certificate{
+		SerialNumber: big.NewInt(2),
+	}
+
+	entityCert, err := x509.CreateCertificate(rand.Reader, rootCA, entityX509, &entityKey.PublicKey, caKey)
+	require.NoError(t, err)
+
+	certFile, err := ioutil.TempFile("", "")
+	require.NoError(t, err)
+	defer certFile.Close()
+
+	// create chained PEM file with CA and entity cert
+	for _, cert := range [][]byte{entityCert, caCert} {
+		require.NoError(t,
+			pem.Encode(certFile, &pem.Block{
+				Type:  "CERTIFICATE",
+				Bytes: cert,
+			}),
+		)
+	}
+
+	keyFile, err := ioutil.TempFile("", "")
+	require.NoError(t, err)
+	defer keyFile.Close()
+
+	entityKeyBytes, err := x509.MarshalECPrivateKey(entityKey)
+	require.NoError(t, err)
+
+	require.NoError(t,
+		pem.Encode(keyFile, &pem.Block{
+			Type:  "ECDSA PRIVATE KEY",
+			Bytes: entityKeyBytes,
+		}),
+	)
+
+	cleanup := func() {
+		require.NoError(t, os.Remove(certFile.Name()))
+		require.NoError(t, os.Remove(keyFile.Name()))
+	}
+
+	return certFile.Name(), keyFile.Name(), cleanup
+}
